<?php
/*
Plugin Name: Network Post Promotion
Description: Výběr článků z blogů v síti a jejich propagace pomocí widgetu. Multisite plugin. <strong>Vyvinuto a určeno pro Pure Art.</strong> 
Version: 1.0
Author: Zaantar
Author URI: http://zaantar.eu
License: GPL2
*/

/*
    Copyright 2011 Zaantar (email: zaantar@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/



/*****************************************************************************\
		LOGGING AND NOTICES
\*****************************************************************************/


define( 'NPP_WLS_LOGNAME', 'network-post-promotion' );

function npp_nag( $message ) {
	echo( '<div id="message" class="updated"><p>'.$message.'</p></div>' );
}


function npp_nagerr( $message ) {
	echo( '<div id="message" class="error"><p>'.$message.'</p></div>' );
}


function npp_log( $message, $category = 1 ) {
	if( !defined( 'WLS' ) or !wls_is_registered( NPP_WLS_LOGNAME ) ) {
		$filename = dirname(__FILE__).'/log.txt';
		$file = fopen( $filename, 'ä́' );
		fwrite( $file, $message );
		fclose( $file );
	} else {
		wls_simple_log( NPP_WLS_LOGNAME, $message, $category );
	}
}


/*****************************************************************************\
		DATABASE
\*****************************************************************************/


register_activation_hook( __FILE__, 'npp_plugin_activation' );

function npp_plugin_activation() {
	global $wpdb;

	npp_log( 'plugin activation triggered' );
	
	$promotions_structure = 
        'CREATE TABLE IF NOT EXISTS '.npp_promotions_table().' (
            id INT NOT NULL AUTO_INCREMENT,
            promotion_time TIMESTAMP DEFAULT NOW() ,
            blog_id BIGINT(20),
            post_id BIGINT(20),
            promoted_by BIGINT(20),
            sticky BOOL,
            UNIQUE ( id ),
            PRIMARY KEY ( id )
        )';
        
	$promotions_ok = $wpdb->query( $wpdb->prepare( $promotions_structure ) );
	if( !$promotions_ok ) {
		npp_log( 'MySQL error while executing query: '.$wpdb->prepare( $promotions_structure ), 5 );
		return false;
	} else {
		npp_log( 'Successfully created table '.npp_promotions_table().': '.$promotions_structure );
	}	

	
}


function npp_promotions_table() {
	global $wpdb;
	return $wpdb->base_prefix.'npp_promotions';
}


function npp_get_promotions( $limit = 'all', $order = 'DESC' ) {
	global $wpdb;
	$query = 'SELECT *
		FROM '.npp_promotions_table().'
		ORDER BY sticky '.$order.', promotion_time '.$order;
	if( $limit != 'all' ) {
		$query.= ' LIMIT '.$limit;
	}
	return $wpdb->get_results( $wpdb->prepare( $query ) );
}


function npp_get_promotion_count() {
	global $wpdb;
	$query = 'SELECT COUNT(*) FROM '.npp_promotions_table();
	return $wpdb->get_var( $wpdb->prepare( $query ) );
}


function npp_delete_promotion( $promotion_id ) {
	global $wpdb;
	$query = 'DELETE FROM '.npp_promotions_table().'
		WHERE id = '.$promotion_id;
	$rt = $wpdb->query( $wpdb->prepare( $query ) );
	if( $rt === FALSE ) {
		npp_log( 'Error while deleting promotion record '.$promotion_id.'.', 4 );
		return false;
	} else {
		npp_log( 'Promotion record '.$promotion_id.' deleted.' );
		return true;
	}
}


function npp_set_sticky_bit( $promotion_id, $value ) {
	global $wpdb;
	$updated = $wpdb->update( npp_promotions_table(), array( 'sticky' => $value ), array( 'id' => $promotion_id ) );
	return ( $updated == 1 );
}


function npp_promote_post( $blog_id, $post_id, $user_id ) {
	global $wpdb;
	$inserted = $wpdb->insert( npp_promotions_table(), 
		array(
			'blog_id' => $blog_id,
			'post_id' => $post_id,
			'promoted_by' => $user_id
		),
		'%d'
	);
	if( $inserted == false ) {
		npp_log( 'Error while promoting post '.$post_id.' on blog '.$blog_id.' by user ID=='.$user_id.'.', 4 );
		return false;
	} else {
		npp_log( 'Post '.$post_id.' on blog '.$blog_id.' promoted by user ID=='.$user_id.'.' );
		npp_maintain_promotion_count();
		return true;
	}
}


function npp_is_promoted( $post_id ) {
	global $blog_id, $wpdb;
	$query = 'SELECT COUNT(*)
		FROM '.npp_promotions_table().'
		WHERE (
			blog_id = '.$blog_id.'
			AND post_id = '.$post_id.'
		)';
	return ( $wpdb->get_var( $wpdb->prepare( $query ) ) > 0 );
}


function npp_maintain_promotion_count() {
	global $wpdb;
	
	$count = npp_get_promotion_count();
	$options = npp_get_options();

	if( $options['records_to_keep'] < $count ) {
		npp_log( 'Maintaining maximum record count.' );
		$last = npp_get_promotions( 1, 'ASC' );
		npp_delete_promotion( $last[0]->id );
	}	
}


/*****************************************************************************\
		OPTIONS
\*****************************************************************************/

add_action( 'network_admin_menu','npp_network_admin_menu' );


function npp_network_admin_menu() {
	add_submenu_page( 'settings.php', 'Nastavení Network Post Promotion', 'Network Post Promotion', 'manage_network_options', 
		'npp-options', 'npp_options_page' );
}


function npp_options_page() {
	if( isset($_REQUEST['action']) ) {
        $action = $_REQUEST['action'];
    } else {
        $action = 'default';
    }
    
    switch( $action ) {
    case 'update-options':
    	$new_options = array(
			'records_to_keep' => $_POST['npp_records_to_keep'],
			'promotion_capability' => $_POST['npp_promotion_capability'],
			'promotion_capability_blog' => $_POST['npp_promotion_capability_blog'],
			'ignored_blogs' => explode( ',', $_POST['npp_ignored_blogs'] )
		);
		$update_ok = npp_update_options( $new_options );
		npp_nag( 'Nastavení byla úspěšně uložena.' );
		npp_options_page_default();
		break;
    case 'wls-register':
    	$ok = wls_register( NPP_WLS_LOGNAME, 'Systémové záznamy pluginu Network Post Promotion.' );
    	if( $ok ) {
    		$info = 'Network Post Promotion byl úspěšně zaregistrován k WLS.';
    		npp_nag( $info );
    		npp_log( $info );
    	} else {
    		$info = 'Při pokusu o registraci Network Post Promotion k WLS došlo k chybě.';
    		npp_nagerr( $info );
    		npp_log( $info, WLS_ERROR );
    	}
    	npp_options_page_default();
    	break;
    case 'wls-unregister':
    	$ok = wls_unregister( NPP_WLS_LOGNAME );
    	if( $ok ) {
    		$info = 'Network Post Promotion byl úspěšně odregistrován z WLS, systémové logy byly smazány.';
    		npp_nag( $info );
    		npp_log( $info );
    	} else {
    		$info = 'Při pokusu o odregistraci Network Post Promotion z WLS došlo k chybě.';
    		npp_nagerr( $info );
    		npp_log( $info, WLS_ERROR );
    	}
    	npp_options_page_default();
    	break;
    case 'delete-promotion':
	    $ok = npp_delete_promotion( $_GET['id'] );
	    if( $ok ) {
	    	npp_nag( 'Záznam byl úspěšně odstraněn.' );
	    } else {
	    	npp_nagerr( 'Při odstraňování záznamu došlo k chybě. Kontaktujte, prosím, webmastera.' );
	    }
	    npp_options_page_default();
    	break;
    case 'sticky':
    	$ok = npp_set_sticky_bit( $_GET['id'], $_GET['value'] );
	    if( $ok ) {
	    	npp_nag( 'Záznam byl úspěšně upraven.' );
	    } else {
	    	npp_nagerr( 'Při upravování záznamu došlo k chybě. Kontaktujte, prosím, webmastera.' );
	    }
	    npp_options_page_default();
    	break;
    default:
    	npp_options_page_default();
    }
}


function npp_options_page_default() {
	$options = npp_get_options();
	?>
	<div id="wrap">
		<h2>Nastavení Network Post Promotion</h2>
		<form method="post" action="settings.php?page=npp-options">
            <input type="hidden" name="action" value="update-options" />
            <table class="form-table">
                <tr valign="top">
                	<th><label for="npp_records_to_keep">Počet článků, které mají být ukládány v databázi</label></th>
                	<td>
                		<input type="text" name="npp_records_to_keep" value="<?php echo $options['records_to_keep']; ?>" />
                	</td>
                </tr>
                <tr valign="top">
                	<th><label for="npp_promotion_capability">Capability, se kterou může uživatel propagovat články</label></th>
                	<td>
                		<input type="text" name="npp_promotion_capability" value="<?php echo $options['promotion_capability']; ?>" />
                	</td>
                </tr>
                <tr valign="top">
                	<th>
                		<label for="npp_promotion_capability_blog">ID blogu, podle kterého se bude kontrolovat capability.</label><br />
                		<small>Zadejte <code>0</code> pro použití aktuálního blogu.</small>
                	</th>
                	<td>
                		<input type="text" name="npp_promotion_capability_blog" value="<?php echo $options['promotion_capability_blog']; ?>" />
                	</td>
                </tr>                		
				<tr valign="top">
                	<th>
                		<label for="npp_ignored_blogs">Blogy, na kterých se nemá zobrazovat tlačítko na propagaci článku.</label><br />
                		<small>Zadejte ID blogů oddělená <code>,</code> (bez mezer).</small>
                	</th>
                	<td>
                		<input type="text" name="npp_ignored_blogs" value="<?php echo implode( ',', $options['ignored_blogs'] ); ?>" />
                	</td>
                </tr>   
			</table>
			<p class="submit">
	            <input type="submit" value="Uložit nastavení" />    
	        </p> 
		</form>
		<?php
			if( defined( 'WLS' ) ) {
				?>			
				<h3>Registrace do systémového logu WLS</h3>
		    	<p>Byl detekován nainstalovaný plugin Wordpress Logging Service.</p>
	    		<?php
	    			if( !wls_is_registered( NPP_WLS_LOGNAME ) ) {
	    				?>
						<form method="post" action="settings.php?page=npp-options">
							<input type="hidden" name="action" value="wls-register" />
							<p class="submit">
								<input type="submit" value="Registrovat Network Post Promotion do WLS" />    
							</p> 
						</form>
						<?php
					} else {
						?>
						<form method="post" action="settings.php?page=npp-options">
							<input type="hidden" name="action" value="wls-unregister" />
							<p class="submit">
								<input type="submit" style="color:red;" value="Odregistrovat Network Post Promotion z WLS a nevratně smazat všechny logy" />    
							</p> 
						</form>
						<?php
					}
				?>
				<?php
        	}
        ?>
        <h3>Seznam propagovaných článků</h3>
        <p>Zde naleznete všechny propagované články uložené v databázi, v pořadí, v jakém se budou zobrazovat ve widgetu.</p>
        <table class="widefat" cellspacing="0">
		    <thead>
		        <tr>
		            <th scope="col" class="manage-column">id</th>
		            <th scope="col" class="manage-column">Čas propagace<br />Propagující uživatel</th>
		            <th scope="col" class="manage-column">Blog<br />"Sticky bit"</th>
		            <th scope="col" class="manage-column">Článek</th>
		            <th scope="col" class="manage-column">Akce</th>
				</tr>
			</thead>
			<tfoot>
				<tr>
		            <th scope="col" class="manage-column">id</th>
		            <th scope="col" class="manage-column">Čas propagace<br />Propagující uživatel</th>
		            <th scope="col" class="manage-column">Blog<br />"Sticky bit"</th>
		            <th scope="col" class="manage-column">Článek</th>
		            <th scope="col" class="manage-column">Akce</th>
				</tr>
			</tfoot>
			<?php
				$promotions = npp_get_promotions();
				foreach( $promotions as $promotion ) {
					?>
					<tr>
						<td><?php echo $promotion->id; ?></td>
						<td>
							<?php echo $promotion->promotion_time; ?><br />
							<?php 
								$promoterdata = get_userdata( $promotion->promoted_by );
								echo $promoterdata->user_login;
							?>
						</td>
						<td>
							<?php 
								 $blog_details = get_blog_details( $promotion->blog_id, true );
								 echo '<a href="http://'.$blog_details->domain.'">'.$blog_details->blogname.'</a>';
							?>
							<br />
							<code>
								<?php
									if( $promotion->sticky ) {
										echo 'TRUE';
									} else {
										echo 'FALSE';
									}
								?>
							</code>
						</td>
						<td>
							<?php
								$post = get_blog_post( $promotion->blog_id, $promotion->post_id );
								echo '<a href="'.get_blog_permalink( $promotion->blog_id, $promotion->post_id ).'">'.$post->post_title.'</a><br /><small>'.substr( $post->post_excerpt, 0, 50 ).'...</small>';
							?>
						</td>
						<td>
							<a href="settings.php?page=npp-options&action=delete-promotion&id=<?php echo $promotion->id; ?>">Odstranit</a><br />
							<?php 
								if( $promotion->sticky ) {
									?>
									<a href="settings.php?page=npp-options&action=sticky&value=0&id=<?php echo $promotion->id; ?>">sticky=0</a>
									<?php
								} else {
									?>
									<a href="settings.php?page=npp-options&action=sticky&value=1&id=<?php echo $promotion->id; ?>">sticky=1</a>
									<?php
								}
							?>
						</td>
					</tr>
					<?php
				}
			?>
		</table>
	</div>
	<?php
}


function npp_get_options() {
	$defaults = array(
		'records_to_keep' => 50,
		'promotion_capability' => 'promote_posts',
		'promotion_capability_blog' => 0,
		'ignored_blogs' => array()
	);
	$options = maybe_unserialize( get_site_option( 'npp_options', $defaults ) );
	return $options;
}


function npp_update_options( $new_options ) {
	$updated = update_site_option( 'npp_options', maybe_serialize( $new_options ) );
	npp_log( 'Options updated: '.print_r( $new_options, true ) );
}


/*****************************************************************************\
		POST PROMOTION
\*****************************************************************************/


add_action( 'comment_form_after', 'npp_comment_form_after' );


function npp_comment_form_after() {
	global $post, $blog_id;
	
	$options = npp_get_options();
	if( $options['promotion_capability_blog'] == 0 ) {
		$options['promotion_capability_blog'] = $blog_id;
	}
	
	if( in_array( $blog_id, $options['ignored_blogs'] ) ) {
		return;
	}
	
	if( !current_user_can_for_blog( $options['promotion_capability_blog'], $options['promotion_capability'] ) ) {
		return;
	}
	
	if( npp_is_promoted( $post->ID ) ) {
		return;
	}
	
	if( isset( $_POST['npp-action'] ) ) {
		$action = $_POST['npp-action'];
	} else {
		$action = 'show-button';
	}
	
	switch( $action ) {
	case 'show-button':	
		?>
		<div id="npp">
			<form method="post">
				<input type="hidden" name="npp-action" value="promote-post" />
				<input type="submit" value="Propagovat článek" />
			</form>
		</div>
		<?
		break;
	case 'promote-post':
		$promoted = npp_promote_post( $blog_id, $post->ID, get_current_user_id() );
		if( $promoted ) {
			?>
			<div id="npp">
				<span class="ok">Propagováno.</span>
			</div>
			<?php
		} else {
			?>
			<div id="npp">
				<span class="error">Chyba, kontaktujte webmastera.</span>
			</div>
			<?php
		}
		break;
	}
}


/*****************************************************************************\
		WIDGET
\*****************************************************************************/


class NPP_Widget extends WP_Widget {
	
	
	// konstruktor
	function NPP_Widget() {
		$widget_ops = array(
			'classname' => 'npp-widget',
			'description' => 'Vypíše seznam příspěvků propagovaných v síti.'
			);
			
		$control_ops = array( 'id_base' => 'npp-widget' );
		$this->WP_Widget( 'npp-widget', 'Propagované články', $widget_ops, $control_ops );
	}
	
	
	// zobrazení widgetu v administraci
	function form( $instance ) {
		
		$defaults = array( 'title' => 'Propagované články', 'post_count' => 10 );
		$instance = wp_parse_args( (array) $instance, $defaults );
		
		?>
		<p>
			<label for="<?php echo $this->get_field_id( 'title' ); ?>">Titulek:</label>
			<input type="text" name="<?php echo $this->get_field_name( 'title' ) ?>" id="<?php echo $this->get_field_id( 'title' ) ?> " value="<?php echo $instance['title'] ?>">
		</p>
		<p>
			<label for="<?php echo $this->get_field_id( 'post_count' ); ?>">Počet zobrazených článků:</label>
			<input type="text" name="<?php echo $this->get_field_name( 'post_count' ) ?>" id="<?php echo $this->get_field_id( 'post_count' ) ?> " value="<?php echo $instance['post_count'] ?>">
		</p>
		<?
		
	}
	
	
	// ukládání změn v administraci
	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;

		$instance['title'] = $new_instance['title'];
		$instance['post_count'] = $new_instance['post_count'];

		return $instance;
	}
	
	
	// zobrazení widgetu na stránce
	function widget( $args, $instance ) {
		extract($args);

		echo $before_widget;
		echo $before_title.$instance['title'].$after_title;
		
		$promotions = npp_get_promotions( $instance['post_count'] );
		
		?>
		<ul>
			<?php
				foreach( $promotions as $promotion ) {
					$post = get_blog_post( $promotion->blog_id, $promotion->post_id );
					$permalink = get_blog_permalink( $promotion->blog_id, $promotion->post_id );
					$blog_details = get_blog_details( $promotion->blog_id, true );
					
					?>
					<li>
						<a href="<?php echo $permalink; ?>"><?php echo $post->post_title; ?></a>
						<strong>@</strong>
						<a href="http://<?php echo $blog_details->domain; ?>"><?php echo $blog_details->blogname; ?></a>
					</li>
					<?php
				}
			?>
		</ul>
		<?php

		echo $after_widget;
	}

}


add_action( 'widgets_init', function() { return register_widget( 'NPP_Widget' ); } );

?>
